/*

Copyright (c) 2001-2004 Fritz Ganter <ganter@ganter.at>
Website: www.gpsdrive.de

Modifications for ipaq specific version
Copyright (c) 2007 Andrew M. Bishop <amb@gedanken.demon.co.uk>
Website: http://www.geganken.demon.co.uk/gpsdrive-ipaq/

Disclaimer: Please do not use for navigation. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    **********************************************************************/


/*  There must be the software "festival" running in server mode */
/*  http://www.speech.cs.cmu.edu/festival */


/*  Include Dateien */
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <math.h>
#include <sys/time.h>
#include "gpsdrive.h"
#include <gmodule.h>
#include <gdk/gdktypes.h>
#include <gtk/gtk.h>
#include "config.h"

#if HAVE_LOCALE_H
#include <locale.h>
#else
# define setlocale(Category, Locale)
#endif
#include "gettext.h"

#include <time.h>

/*  Defines for gettext I18n */
# include <libintl.h>
# define _(String) gettext(String)
# ifdef gettext_noop
#  define N_(String) gettext_noop(String)
# else
#  define N_(String) (String)
# endif

extern gint debug, havespeechout, posmode, muteflag;
gchar *displaytext = NULL;
extern enum
{ english, german, spanish }
voicelang;
extern GdkColor mygray, blue;
extern GdkDrawable *drawable;
extern gint real_screen_y, real_screen_x;
gint do_display_dsc = FALSE, textcount;
extern gint milesflag,foundradar,speechcount,havepos;
extern gchar targetname[40], oldangle[100];
extern gdouble dist,bearing,groundspeed;
extern GtkWidget *drawing_area;
extern int sound_direction, sound_distance, sound_speed, sound_gps;


#define SPEECHOUTSERVER "127.0.0.1"


gint
speech_out_speek (char *text)
{
  gint e;
  gchar out[2000];
  gint file;

  if (!havespeechout)
    return 0;
  if (posmode)
    return 0;

  if (debug)
    g_print (text);

    {
      file = creat ("/tmp/gpsdrivetext.out", 0666);
      if (file == -1)
	return 0;
      e = write (file, text, strlen (text));
      e = write (file, ".\n", 2);
      close (file);
      g_snprintf (out, sizeof (out), "flite -f /tmp/gpsdrivetext.out&");
      system (out);
    }
  return e;
}

/*  if second parameter is TRUE, then also greeting is spoken */
gint
speech_saytime_cb (GtkWidget * widget, guint datum)
{
  time_t t;
  struct tm *ts;
  gchar buf[200];

  if (muteflag)
    return TRUE;

  time (&t);
  ts = localtime (&t);

  if (havespeechout)
    switch (voicelang)
      {
      case english:
	if (datum == 1)
	  {
	    if ((ts->tm_hour >= 0) && (ts->tm_hour < 12))
	      g_snprintf (buf, sizeof (buf), "Good Morning,");
	    if ((ts->tm_hour >= 12) && (ts->tm_hour < 18))
	      g_snprintf (buf, sizeof (buf), "Good afternoon,");
	    if (ts->tm_hour >= 18)
	      g_snprintf (buf, sizeof (buf), "Good evening,");
	    speech_out_speek (buf);
	  }
	g_snprintf (buf, sizeof (buf),
		    "it is %d,%02d.", ts->tm_hour,
		    ts->tm_min);
	speech_out_speek (buf);
	break;
      case spanish:
	if (datum == 1)
	  {
	    if ((ts->tm_hour >= 0) && (ts->tm_hour < 12))
	      g_snprintf (buf, sizeof (buf), "buenos das.");
	    if ((ts->tm_hour >= 12) && (ts->tm_hour < 18))
	      g_snprintf (buf, sizeof (buf), "buenos tardes.");
	    if (ts->tm_hour >= 18)
	      g_snprintf (buf, sizeof (buf), "buenas noches");
	    speech_out_speek (buf);
	  }
	if (ts->tm_hour == 1)
	  g_snprintf (buf, sizeof (buf), "es la una y %d minutos.",
		      ts->tm_min);
	else
	  g_snprintf (buf, sizeof (buf),
		      "son las %d horas y %d minutos.", ts->tm_hour,
		      ts->tm_min);
	speech_out_speek (buf);
	break;
      case german:
	if (datum == 1)
	  {
	    if ((ts->tm_hour >= 0) && (ts->tm_hour < 12))
	      g_snprintf (buf, sizeof (buf), "Guten Morgen.");
	    if ((ts->tm_hour >= 12) && (ts->tm_hour < 18))
	      g_snprintf (buf, sizeof (buf), "Guten Tag.");
	    if (ts->tm_hour >= 18)
	      g_snprintf (buf, sizeof (buf), "Guten Abend.");
	    speech_out_speek (buf);
	  }
	if (ts->tm_hour == 1)
	  g_snprintf (buf, sizeof (buf), "es ist ein Uhr %d", ts->tm_min);
	else
	  g_snprintf (buf, sizeof (buf), "es ist %d Uhr %d", ts->tm_hour,
		      ts->tm_min);
	speech_out_speek (buf);

      }

  return TRUE;

}

void
saytargettext (gchar * filename, gchar * tg)
{
  gchar file[500];
  gint fd, e;
  gchar *start, *end;
  struct stat buf;
  gchar *data, *b, *tg2, target[100];

/* build .dsc filename */
  g_strlcpy (file, filename, sizeof (file));
  file[strlen (file) - 3] = 0;
  g_strlcat (file, "dsc", sizeof (file));
/* get size */
  e = stat (file, &buf);
  if (e != 0)
    return;

  fd = open (file, O_RDONLY);
/* map +2000 bytes to get 0 at the end */
  data = mmap (0, buf.st_size + 2000, PROT_READ, MAP_SHARED, fd, 0);
  g_strlcpy (target, "$", sizeof (target));
  tg2 = g_strdelimit (tg, " ", '_');
  g_strlcat (target, tg2, sizeof (target));
  start = strstr (data, target);
  if (start != NULL)
    {
      start = strstr (start, "\n");
      end = strstr (start, "$");
      if (end == NULL)
	end = start + strlen (start);
      b = calloc (end - start + 50, 1);
      if (displaytext != NULL)
	free (displaytext);
      displaytext = calloc (end - start + 50, 1);
      strncpy (displaytext, start, end - start);
      displaytext[end - start + 1] = 0;
      g_strlcpy (b, displaytext, end - start + 50);
      displaytext = g_strdelimit (displaytext, "\n", ' ');
      do_display_dsc = TRUE;
      textcount = 0;
      speech_out_speek (b);
      free (b);
    }
  munmap (data, buf.st_size + 2000);
}

void
display_dsc (void)
{
  GdkGC *kontext;
  gint len;
  gchar *text;
  PangoFontDescription *pfd;
  PangoLayout *wplabellayout;


  if (!do_display_dsc)
    return;
  if ((textcount >= (int) strlen (displaytext)))
    {
      do_display_dsc = FALSE;
      free (displaytext);
      displaytext = NULL;
      return;
    }
  if (textcount > 20)
    text = displaytext + textcount;
  else
    text = displaytext;
  kontext = gdk_gc_new (drawable);
  len = strlen (text);
/*   if (len>10) */
/*       len=10; */


/*   gdk_gc_set_function (kontext, GDK_OR); */

  gdk_gc_set_foreground (kontext, &mygray);
  gdk_draw_rectangle (drawable, kontext, 1, 0, SCREEN_Y - 40, SCREEN_X, 40);
  gdk_gc_set_function (kontext, GDK_COPY);
/*   gdk_gc_set_foreground (kontext, &blue); */
/*   gdk_draw_text (drawable, bigtextfont, kontext, */
/* 		 11, SCREEN_Y - 30, text, len); */

  /* prints in pango */

  wplabellayout = gtk_widget_create_pango_layout (drawing_area, text);
//KCFX  
  pfd = pango_font_description_from_string ("Sans 8");

  pango_layout_set_font_description (wplabellayout, pfd);
/* 		pango_layout_get_pixel_size (wplabellayout, &width, &height); */
  gdk_draw_layout_with_colors (drawable, kontext, 11, SCREEN_Y - 30,
			       wplabellayout, &blue, NULL);

  if (wplabellayout != NULL)
    g_object_unref (G_OBJECT (wplabellayout));
  /* freeing PangoFontDescription, cause it has been copied by prev. call */
  pango_font_description_free (pfd);



  textcount += 2;

}

gint
speech_out_cb (GtkWidget * widget, guint * datum)
{
  gchar buf[500], s2[100];
  gint angle;

  if (strcmp (oldangle, "XXX"))
    {
      if (muteflag)
	return TRUE;
      if (foundradar)
	return TRUE;
    }

  speechcount++;
  angle = bearing * 180.0 / M_PI;

  if (!havepos)
    {
      switch (voicelang)
	{
	case english:
	  g_snprintf (buf, sizeof (buf), "Not enough satellites in view");
	  break;
	case spanish:
	  g_snprintf (buf, sizeof (buf), "El GPS Fix no est disponible");
	  break;
	case german:
	  g_snprintf (buf, sizeof (buf), "Zuwenig Satelliten in Sicht");
	}
      if ((speechcount == 1) && sound_gps)
	speech_out_speek (buf);
      return TRUE;
    }


  switch (voicelang)
    {
    case english:
      if ((angle >= 338) || (angle < 22))
	g_strlcpy (s2, "in front of you", sizeof (s2));
      if ((angle >= 22) && (angle < 68))
	g_strlcpy (s2, "ahead of you to the right", sizeof (s2));
      if ((angle >= 68) && (angle < 112))
	g_strlcpy (s2, "to your right", sizeof (s2));
      if ((angle >= 112) && (angle < 158))
	g_strlcpy (s2, "behind you to the right", sizeof (s2));
      if ((angle >= 158) && (angle < 202))
	g_strlcpy (s2, "behind you", sizeof (s2));
      if ((angle >= 202) && (angle < 248))
	g_strlcpy (s2, "behind you to the left", sizeof (s2));
      if ((angle >= 248) && (angle < 292))
	g_strlcpy (s2, "to your left", sizeof (s2));
      if ((angle >= 292) && (angle < 338))
	g_strlcpy (s2, "ahead of you to the left", sizeof (s2));
      break;
    case spanish:
      if ((angle >= 338) || (angle < 22))
	g_strlcpy (s2, "delante de usted", sizeof (s2));
      if ((angle >= 22) && (angle < 68))
	g_strlcpy (s2, "delante de usted a la derecha", sizeof (s2));
      if ((angle >= 68) && (angle < 112))
	g_strlcpy (s2, "a la derecha", sizeof (s2));
      if ((angle >= 112) && (angle < 158))
	g_strlcpy (s2, "de tras de usted a la derecha", sizeof (s2));
      if ((angle >= 158) && (angle < 202))
	g_strlcpy (s2, "de tras de usted", sizeof (s2));
      if ((angle >= 202) && (angle < 248))
	g_strlcpy (s2, "de tras de usted a la izquierda", sizeof (s2));
      if ((angle >= 248) && (angle < 292))
	g_strlcpy (s2, "a la izquierda", sizeof (s2));
      if ((angle >= 292) && (angle < 338))
	g_strlcpy (s2, "delante de usted a la izquierda", sizeof (s2));
      break;
    case german:
      if ((angle >= 338) || (angle < 22))
	g_strlcpy (s2, "vor ihnen", sizeof (s2));

      if ((angle >= 22) && (angle < 68))
	g_strlcpy (s2, "rechts vor ihnen", sizeof (s2));
      if ((angle >= 68) && (angle < 112))
	g_strlcpy (s2, "rechts", sizeof (s2));
      if ((angle >= 112) && (angle < 158))
	g_strlcpy (s2, "rechts hinter ihnen", sizeof (s2));
      if ((angle >= 158) && (angle < 202))
	g_strlcpy (s2, "hinter ihnen", sizeof (s2));
      if ((angle >= 202) && (angle < 248))
	g_strlcpy (s2, "links hinter ihnen", sizeof (s2));
      if ((angle >= 248) && (angle < 292))
	g_strlcpy (s2, "links", sizeof (s2));
      if ((angle >= 292) && (angle < 338))
	g_strlcpy (s2, "links vor ihnen", sizeof (s2));
    }
  if ((speechcount == 1) || (strcmp (s2, oldangle)))
    {
      switch (voicelang)
	{
	case english:
	  g_snprintf (buf, sizeof (buf), "Destination is %s", s2);
	  break;
	case spanish:
	  g_snprintf (buf, sizeof (buf), "Su destinacin est %s", s2);
	  break;
	case german:
	  g_snprintf (buf, sizeof (buf), "Das Ziel ist %s", s2);
	}
      if (sound_direction)
      speech_out_speek (buf);
      g_strlcpy (oldangle, s2, sizeof (oldangle));
    }
  if (speechcount == 3 && groundspeed >= 20)
    {
      switch (voicelang)
	{
	case english:
	  if (milesflag)
	    g_snprintf (buf, sizeof (buf),
			"The current speed is %d miles per hour",
			(int) (groundspeed));
	  else
	    g_snprintf (buf, sizeof (buf),
			"The current speed is %d kilometers per hour",
			(int) groundspeed);
	  break;
	case spanish:
	  if (milesflag)
	    g_snprintf (buf, sizeof (buf),
			"La velocidad actual es %d milla por hora",
			(int) (groundspeed));
	  else
	    g_snprintf (buf, sizeof (buf),
			"La velocidad actual es %d kilometros por hora",
			(int) groundspeed);
	  break;
	case german:
	  if (milesflag)
	    g_snprintf (buf, sizeof (buf),
			"Die momentane Geschwindigkeit ist %d Meilen pro Stunde",
			(int) (groundspeed));
	  else
	    g_snprintf (buf, sizeof (buf),
			"Die momentane Geschwindigkeit ist %d kmh",
			(int) groundspeed);
	}
      if (sound_speed)
      speech_out_speek (buf);
    }

  if (speechcount > 10)
    speechcount = 0;


  if ((speechcount == 2) || ((dist < 1.2) && (speechcount == 7)))
    {
      if (milesflag)
	{
	  switch (voicelang)
	    {
	    case english:
	      if (dist <= 1.2)
		g_snprintf (s2, sizeof (s2), "%.0f yards", dist * 1760.0);
	      else
		g_snprintf (s2, sizeof (s2), "%.0f miles", dist);
	      break;
	    case spanish:
	      if (dist <= 1.2)
		g_snprintf (s2, sizeof (s2), "%.0f yards", dist * 1760.0);
	      else
		g_snprintf (s2, sizeof (s2), "%.0f millas", dist);
	      break;
	    case german:
	      if (dist <= 1.2)
		g_snprintf (s2, sizeof (s2), "%.0f yard", dist * 1760.0);
	      else
		g_snprintf (s2, sizeof (s2), "%.0f Meilen", dist);
	    }
	}
      else
	{
	  switch (voicelang)
	    {
	    case english:
	      if (dist <= 1.2)
		g_snprintf (s2, sizeof (s2), "%.0f meters", dist * 1000.0);
	      else
		g_snprintf (s2, sizeof (s2), "%.0f kilometers", dist);
	      break;
	    case spanish:
	      if (dist <= 1.2)
		g_snprintf (s2, sizeof (s2), "%.0f metros", dist * 1000.0);
	      else
		g_snprintf (s2, sizeof (s2), "%.0f kilometros", dist);
	      break;
	    case german:
	      if (dist <= 1.2)
		g_snprintf (s2, sizeof (s2), "%.0f meter", dist * 1000.0);
	      else if ((int) dist == 1)
		g_snprintf (s2, sizeof (s2), "ein kilometer");
	      else
		g_snprintf (s2, sizeof (s2), "%d kilometer", (int) dist);
	    }
	}

      switch (voicelang)
	{
	case english:
	  g_snprintf (buf, sizeof (buf), "Distance to %s is %s", targetname,
		      s2);
	  break;
	case spanish:
	  g_snprintf (buf, sizeof (buf), "La distancia a la %s es %s",
		      targetname, s2);
	  break;
	case german:
	  g_snprintf (buf, sizeof (buf), "Die Entfernung bis %s ist %s",
		      targetname, s2);
	}
      if (sound_distance)
      speech_out_speek (buf);
    }

  return TRUE;
}
